@@ -1,25 +1,23 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
-from .models import Place, Person, Institution, Event, Work
-from apis_core.apis_metainfo.models import Uri, Collection
-from apis_core.default_settings.NER_settings import autocomp_settings as ac_settings
-from django.conf import settings
-from django.core.exceptions import FieldError
-from .custom_autocompletes import *
-
-from dal import autocomplete
-from django.db.models import Q
-from django import http
-from django.contrib.contenttypes.models import ContentType
-
-import requests
 import json
 import operator
-from functools import reduce
-import dateutil.parser
 import re
+from functools import reduce
 
+import dateutil.parser
+import requests
+from dal import autocomplete
+from django import http
+from django.conf import settings
+from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import FieldError
+from django.db.models import Q
+
+from apis_core.apis_metainfo.models import Uri, Collection
 from apis_core.apis_vocabularies.models import VocabsBaseClass
+from apis_core.default_settings.NER_settings import autocomp_settings as ac_settings
+from .models import AbstractEntity
 
 
 class CustomEntityAutocompletes(object):
@@ -116,19 +114,17 @@ class GenericEntitiesAutocomplete(autocomplete.Select2ListView):
         db_include = self.kwargs.get('db_include', False)
         choices = []
         headers = {'Content-Type': 'application/json'}
-        ent_model = ContentType.objects.get(
-            app_label='apis_entities', model=ac_type.lower()
-        ).model_class()
+        ent_model = AbstractEntity.get_entity_class_of_name(ac_type)
         if self.q.startswith('http'):
             res = ent_model.objects.filter(uri__uri=self.q.strip())
-        else:
+        elif len(self.q) > 0:
             q1 = re.match('^([^\[]+)\[([^\]]+)\]$', self.q)
             if q1:
                 q = q1.group(1).strip()
                 q3 = q1.group(2).split(',')
                 q3 = [e.strip() for e in q3]
             else:
-                q = re.match('^[^[]+', self.q).group(0)
+                q = re.match('^[^\[]+', self.q).group(0)
                 q3 = False
             if re.match('^[^*]+\*$', q.strip()):
                 search_type = '__istartswith'
@@ -136,13 +132,15 @@ class GenericEntitiesAutocomplete(autocomplete.Select2ListView):
             elif re.match('^\*[^*]+$', q.strip()):
                 search_type = '__iendswith'
                 q = re.match('^\*([^*]+)$', q.strip()).group(1)
-            elif re.match('^\*[^*]+\*$', q.strip()):
-                search_type = '__icontains'
-                q = re.match('^\*([^*]+)\*$', q.strip()).group(1)
-            elif re.match('^[^*]+$', q.strip()):
+            elif re.match('^\"[^"]+\"$', q.strip()):
                 search_type = ''
+                q = re.match('^\"([^"]+)\"$', q.strip()).group(1)
+            elif re.match('^[^*]+$', q.strip()):
+                search_type = '__icontains'
+                q = q.strip()
+            else:
+                search_type = '__icontains'
                 q = q.strip()
-
             arg_list = [Q(**{x+search_type: q}) for x in settings.APIS_ENTITIES[ac_type.title()]['search']]
             res = ent_model.objects.filter(reduce(operator.or_, arg_list)).distinct()
             if q3:
@@ -153,6 +151,9 @@ class GenericEntitiesAutocomplete(autocomplete.Select2ListView):
                     res = res.filter(**f_dict2)
                 except Exception as e:
                     choices.append({'name': str(e)})
+        else:
+            q = ''
+            res = []
         test_db = True
         test_stanbol = False
         test_stanbol_list = dict()
@@ -332,13 +333,13 @@ class GenericVocabulariesAutocomplete(autocomplete.Select2ListView):
         vocab_model = ContentType.objects.get(app_label='apis_vocabularies', model=vocab).model_class()
         if direct == 'normal':
             if vocab_model.__bases__[0] == VocabsBaseClass:
-                choices = [{'id': x.pk, 'text': x.label} for x in vocab_model.objects.filter(name__icontains=q).order_by('name')[offset:offset+page_size]]
+                choices = [{'id': x.pk, 'text': x.label} for x in vocab_model.objects.filter(name__icontains=q).order_by('parent_class__name', 'name')[offset:offset+page_size]]
             else:
                 choices = [{'id': x.pk, 'text': x.label} for x in vocab_model.objects.filter(
-                    Q(name__icontains=q) | Q(name_reverse__icontains=q)).order_by('name')[offset:offset+page_size]]
+                    Q(name__icontains=q) | Q(name_reverse__icontains=q)).order_by('parent_class__name', 'name')[offset:offset+page_size]]
         elif direct == 'reverse':
             choices = [{'id': x.pk, 'text': x.label_reverse} for x in vocab_model.objects.filter(
-                Q(name__icontains=q) | Q(name_reverse__icontains=q)).order_by('name')[offset:offset+page_size]]
+                Q(name__icontains=q) | Q(name_reverse__icontains=q)).order_by('parent_class__name', 'name')[offset:offset+page_size]]
         if len(choices) == page_size:
             more = True
         return http.HttpResponse(json.dumps({
@@ -354,6 +355,24 @@ class GenericNetworkEntitiesAutocomplete(autocomplete.Select2ListView):
         if q.startswith('cl:'):
             res = Collection.objects.filter(name__icontains=q[3:])
             results = [{'id': 'cl:'+str(x.pk), 'text': x.name} for x in res]
+        elif q.startswith('reg:'):
+            results = []
+            if entity.lower() == 'person':
+                filen = 'reg_persons.json'
+            elif entity.lower() == 'place':
+                filen = 'reg_places.json'
+            with open(filen, 'r') as reg:
+                r1 = json.load(reg)
+                r_dict = dict()
+                for r2 in r1:
+                    if q[4:].lower() in r2[1].lower():
+                        if r2[1] in r_dict.keys():
+                            r_dict[r2[1]] += "|{}".format(r2[0])
+                        else:
+                            r_dict[r2[1]] = r2[0]
+            for k in r_dict.keys():
+                results.append({'id': 'reg:'+r_dict[k], 'text': k})
+
         else:
             ent_model = ContentType.objects.get(
                 app_label__startswith='apis_', model=entity
